@@ -14,11 +14,11 @@ function(l1_formula = 'NA', l2_formula = 'NA', data, id, l1_hyper, l2_hyper, bur
   # check each column in the dataframe should have the class 'factor' or 'numeric', no other classes such as 'matrix'...
   for (i in 1:ncol(data)){
     if(class(data[,i]) != 'factor' && class(data[,i]) != 'numeric' && class(data[,i]) != 'integer') stop("data class must be 'factor', 'numeric' or 'integer'")
-    response_name <- attr(mf1,"names")[attr(attr(mf1, "terms"),"response")]
-    # checking missing predictors
-    if(i != which(colnames(data) == response_name) & sum(is.na(data[,i])) > 0) stop("Data type error, NAs/missing values included in independent variables") 
-    if(i != which(colnames(data) == response_name) & class(data[,i]) == 'numeric')
-      data[,i] = data[,i] - mean(data[,i])
+    #response_name <- attr(mf1,"names")[attr(attr(mf1, "terms"),"response")]
+    # checking missing predictors, already checked in design matrix
+    # if(i != which(colnames(data) == response_name) & sum(is.na(data[,i])) > 0) stop("Data type error, NAs/missing values included in independent variables") 
+    #if(i != which(colnames(data) == response_name) & class(data[,i]) == 'numeric')
+    #  data[,i] = data[,i] - mean(data[,i])
   }
   
   n <- nrow(data)
@@ -45,6 +45,8 @@ function(l1_formula = 'NA', l2_formula = 'NA', data, id, l1_hyper, l2_hyper, bur
     samples_l2_param <- result$mcmc[[1]][,index_l2_param]
   else
     samples_l2_param <- matrix(result$mcmc[[1]][,index_l2_param], ncol = 1)
+  colnames(samples_l2_param) <- colnames(result$mcmc[[1]])[index_l2_param]
+  
   n_p_l1 <- length(JAGS.model$monitorl1.parameters)
   index_l1_param<- array(0,dim = c(n_p_l1,1))
   for (i in 1:n_p_l1)
@@ -53,6 +55,7 @@ function(l1_formula = 'NA', l2_formula = 'NA', data, id, l1_hyper, l2_hyper, bur
     samples_l1_param <- result$mcmc[[1]][,index_l1_param]
   else
     samples_l1_param <- matrix(result$mcmc[[1]][,index_l1_param], ncol = 1)
+  colnames(samples_l1_param) <- colnames(result$mcmc[[1]])[index_l1_param]
   
   #anova.table <- table.ANOVA(samples_l1_param, dMatrice$X, dMatrice$Z)
   cat('Constructing ANOVA/ANCOVA tables...\n')
