@@ -1,5 +1,5 @@
 multi.print.table.means <-
-function (coeff_table, n_choice , samples_l2_param, X_assign = array(dim = 0), X_classes = character(0), Z_assign = array(dim = 0), Z_classes = character(0), 
+function (coeff_table, n_choice, samples_l2_param, X_names, X_assign = array(dim = 0), X_classes = character(0), Z_names, Z_assign = array(dim = 0), Z_classes = character(0), 
                                l1_values = list(), l1_interactions = list(), l1_interactions_index = array(dim = 0), 
                                l2_values = list(), l2_interactions = list(), l2_interactions_index = array(dim = 0), 
                                numeric_index_in_X, numeric_index_in_Z){
@@ -8,7 +8,7 @@ function (coeff_table, n_choice , samples_l2_param, X_assign = array(dim = 0), X
   
   num_l1 <- length(X_assign)
   num_l2 <- length(Z_assign)
-  est_matrix <- array(0 , dim = c(num_l1, num_l2, n_sample))
+  est_matrix <- array(0 , dim = c(num_l1, num_l2, n_sample), dimnames = list(X_names, Z_names, NULL))
   for (i in 1:num_l1){
     for (j in 1:n_sample)
       est_matrix[i,,j] <- samples_l2_param[j,((i-1)*num_l2+1):((i-1)*num_l2+num_l2)]
@@ -47,7 +47,7 @@ function (coeff_table, n_choice , samples_l2_param, X_assign = array(dim = 0), X
       l1_matrix <- list()
       #l2_v <- matrix(c(1, rep(0, num_l2 - 1)), nrow = 1) # only look at the intercept in level 2
       for (i in 1:length(l1_factors)){
-        l1_matrix[[i]] <- multi.effect.matrix.factor(n_choice, levels(l1_values[[l1_factors[i]]]), X_assign, l1_factors[i], numeric_index_in_X)
+        l1_matrix[[i]] <- multi.effect.matrix.factor(n_choice, l1_values[[l1_factors[i]]], X_assign, l1_factors[i], numeric_index_in_X)
         # Compute median and quantile
         est_samples <- est.multi(est_matrix, n_sample, l1_matrix[[i]], l2_v)
         est_l1mean <- 0
@@ -79,7 +79,7 @@ function (coeff_table, n_choice , samples_l2_param, X_assign = array(dim = 0), X
       cat('Table of means of for factors at level 2: \n')
       l2_matrix <- list()
       for (i in 1:length(l2_factors)){
-        l2_matrix[[i]] <- effect.matrix.factor(levels(l2_values[[l2_factors[i]]]), Z_assign, l2_factors[i], numeric_index_in_Z)
+        l2_matrix[[i]] <- effect.matrix.factor(l2_values[[l2_factors[i]]], Z_assign, l2_factors[i], numeric_index_in_Z)
         est_samples <- est.multi(est_matrix, n_sample, l1_v, l2_matrix[[i]])
         est_l2mean <- 0
         cat('\nFactor:',attr(Z_classes, 'names')[l2_factors[i]],'\n')
@@ -146,11 +146,11 @@ function (coeff_table, n_choice , samples_l2_param, X_assign = array(dim = 0), X
     index <- 1
     for (i in 1:length(l1_interactions)){
       temp1 <- l1_values[l1_interactions[[i]]]
-      if (length(temp1) == 2){
-        temp2 <- list()
-        for (j in 1:length(temp1))
-          temp2[[j]] <- levels(temp1[[j]])
-        l1_inter_matrix[[index]] <- multi.effect.matrix.interaction(n_choice, interaction_factors = temp2, assign = X_assign, 
+      if (length(temp1) >= 2){
+        #temp2 <- list()
+        #for (j in 1:length(temp1))
+          #temp2[[j]] <- levels(temp1[[j]])
+        l1_inter_matrix[[index]] <- multi.effect.matrix.interaction(n_choice, interaction_factors = temp1, assign = X_assign, 
                                                                     l1_interactions[[i]], index_inter_factor = l1_interactions_index[i], 
                                                                     numeric_index_in_X)
         est_l1inmean <- 0
@@ -219,11 +219,11 @@ function (coeff_table, n_choice , samples_l2_param, X_assign = array(dim = 0), X
     index <- 1
     for (i in 1:length(l2_interactions)){
       temp1 <- l2_values[l2_interactions[[i]]]
-      if (length(temp1) == 2){
-        temp2 <- list()
-        for (j in 1:length(temp1))
-          temp2[[j]] <- levels(temp1[[j]])
-        l2_inter_matrix[[index]] <- effect.matrix.interaction(interaction_factors = temp2, assign = Z_assign, 
+      if (length(temp1) >= 2){
+        #temp2 <- list()
+        #for (j in 1:length(temp1))
+          #temp2[[j]] <- levels(temp1[[j]])
+        l2_inter_matrix[[index]] <- effect.matrix.interaction(interaction_factors = temp1, assign = Z_assign, 
                                                               l2_interactions[[i]], index_inter_factor = l2_interactions_index[i], 
                                                               numeric_index_in_Z)
         est_l2inmean <- 0
@@ -314,7 +314,7 @@ function (coeff_table, n_choice , samples_l2_param, X_assign = array(dim = 0), X
       l1_matrix <- list()
       #l2_v <- matrix(c(1, rep(0, num_l2 - 1)), nrow = 1) # only look at the intercept in level 2
       for (i in 1:length(l1_factors)){
-        l1_matrix[[i]] <- multi.effect.matrix.factor(n_choice, levels(l1_values[[l1_factors[i]]]), X_assign, l1_factors[i], numeric_index_in_X)
+        l1_matrix[[i]] <- multi.effect.matrix.factor(n_choice, l1_values[[l1_factors[i]]], X_assign, l1_factors[i], numeric_index_in_X)
         # Compute median and quantile
         est_samples <- est.multi(est_matrix, n_sample, l1_matrix[[i]], l2_v)
         cat('\nFactor:',attr(X_classes, 'names')[l1_factors[i]],'\n')
@@ -346,7 +346,7 @@ function (coeff_table, n_choice , samples_l2_param, X_assign = array(dim = 0), X
       cat('Means for factors at level 2: \n')
       l2_matrix <- list()
       for (i in 1:length(l2_factors)){
-        l2_matrix[[i]] <- effect.matrix.factor(levels(l2_values[[l2_factors[i]]]), Z_assign, l2_factors[i], numeric_index_in_Z)
+        l2_matrix[[i]] <- effect.matrix.factor(l2_values[[l2_factors[i]]], Z_assign, l2_factors[i], numeric_index_in_Z)
         est_samples <- est.multi(est_matrix, n_sample, l1_v, l2_matrix[[i]])
         cat('\nFactor:',attr(Z_classes, 'names')[l2_factors[i]],'\n')
         for (n_c in 1: n_choice){
@@ -415,11 +415,11 @@ function (coeff_table, n_choice , samples_l2_param, X_assign = array(dim = 0), X
     index <- 1
     for (i in 1:length(l1_interactions)){
       temp1 <- l1_values[l1_interactions[[i]]]
-      if (length(temp1) == 2){
-        temp2 <- list()
-        for (j in 1:length(temp1))
-          temp2[[j]] <- levels(temp1[[j]])
-        l1_inter_matrix[[index]] <- multi.effect.matrix.interaction(n_choice, interaction_factors = temp2, assign = X_assign, 
+      if (length(temp1) >= 2){
+        #temp2 <- list()
+        #for (j in 1:length(temp1))
+          #temp2[[j]] <- levels(temp1[[j]])
+        l1_inter_matrix[[index]] <- multi.effect.matrix.interaction(n_choice, interaction_factors = temp1, assign = X_assign, 
                                                               l1_interactions[[i]], index_inter_factor = l1_interactions_index[i], 
                                                               numeric_index_in_X) 
         est_samples <- est.multi(est_matrix, n_sample, l1_inter_matrix[[index]], l2_v)
@@ -490,11 +490,11 @@ function (coeff_table, n_choice , samples_l2_param, X_assign = array(dim = 0), X
     index <- 1
     for (i in 1:length(l2_interactions)){
       temp1 <- l2_values[l2_interactions[[i]]]
-      if (length(temp1) == 2){
-        temp2 <- list()
-        for (j in 1:length(temp1))
-          temp2[[j]] <- levels(temp1[[j]])
-        l2_inter_matrix[[index]] <- effect.matrix.interaction(interaction_factors = temp2, assign = Z_assign, 
+      if (length(temp1) >= 2){
+        #temp2 <- list()
+        #for (j in 1:length(temp1))
+          #temp2[[j]] <- levels(temp1[[j]])
+        l2_inter_matrix[[index]] <- effect.matrix.interaction(interaction_factors = temp1, assign = Z_assign, 
                                                               l2_interactions[[i]], index_inter_factor = l2_interactions_index[i], 
                                                               numeric_index_in_Z) 
         est_samples <- est.multi(est_matrix, n_sample, l1_v, l2_inter_matrix[[index]])
